@@ -64,7 +64,10 @@ class Function(Proxy):
                 self.optional_params.append((self.params[-1 - i], defaults[-1 - i]))
         else:
             self.required_params = self.params
-            self.optional_params = []
+            self.optional_params = ()
+        self.params = tuple(self.params)
+        self.required_params = tuple(self.required_params)
+        self.optional_params = tuple(self.optional_params)
             
     def _resolve_args(self, *args, **kw):
         d = {name: default for name, default in self.optional_params}
@@ -79,13 +82,32 @@ class Function(Proxy):
     
     def _target(self):
         return self._func
+    
+def PartialFunction(func, init_args=(), init_kw=None, call_args=(), call_kw=None):
+    if init_kw is None:
+        init_kw = {}
+    if call_kw is None:
+        call_kw = {}
+    class _PartialFunction(func):
+        def _init(self, *args, **kw):
+            args = tuple(init_args) + args
+            kw.update(init_kw)
+            super(_PartialFunction, self)._init(*args, **kw)
             
-class BoundedFunction(Function):
-    def __init__(self, func, instance):
-        super(BoundedFunction, self).__init__(func)
-        self._instance = instance
+        def _call(self, *args, **kw):
+            args = tuple(call_args) + args
+            merged_kw = dict(call_kw)
+            merged_kw.update(kw)
+            return super(_PartialFunction, self)._call(*args, **merged_kw)
+            
+        def _parse_params(self, func):
+            super(_PartialFunction, self)._parse_params(func)
+            self.params = self.params[len(call_args):]
+            self.required_params = self.params[len(call_args):]
+            self.params = tuple([p for p in self.params if p not in call_kw])
+            self.required_params = tuple([p for p in self.required_params if p not in call_kw])
+            self.optional_params = tuple([(k, v) for (k, v) in self.optional_params if k not in call_kw])
+    return _PartialFunction
         
-    def __call__(self, *args, **kw):
-        args = [self._instance] + list(args)
-        return self._func(*args, **kw)
-        
\ No newline at end of file
+def BoundedFunction(func, instance):
+    return PartialFunction(Function, call_args=(instance,))(func)
